@@ -3,6 +3,7 @@
 from typing import Generator
 import warnings
 
+from hypothesis import HealthCheck, Verbosity, settings
 import pytest
 
 try:
@@ -12,6 +13,34 @@ try:
 except ImportError:
     COVERAGE_AVAILABLE = False
 
+# Configure hypothesis settings globally
+settings.register_profile(
+    "default",
+    max_examples=100,
+    deadline=None,
+    suppress_health_check=[HealthCheck.too_slow],
+    verbosity=Verbosity.normal,
+)
+
+settings.register_profile(
+    "ci",
+    max_examples=200,
+    deadline=None,
+    suppress_health_check=[HealthCheck.too_slow],
+    verbosity=Verbosity.verbose,
+)
+
+settings.register_profile(
+    "dev",
+    max_examples=20,
+    deadline=None,
+    suppress_health_check=[HealthCheck.too_slow],
+    verbosity=Verbosity.normal,
+)
+
+# Load default profile
+settings.load_profile("default")
+
 
 @pytest.fixture(autouse=True)
 def suppress_test_warnings() -> Generator[None, None, None]:
