@@ -827,6 +827,44 @@ CALL ECHO Test
         finally:
             os.unlink(temp_file)
 
+    def test_call_environment_variables(self) -> None:
+        """Test CALL with environment variables (should not trigger E014)."""
+        content = """@echo off
+REM Test case from GitHub issue - environment variable as executable
+CALL "%@EMERGENCY_JOB%"
+REM Other environment variable patterns
+CALL "%VARIABLE%"
+CALL "%PATH_VAR%"
+CALL %MY_SCRIPT%
+CALL "%MY_APP%\\script.bat"
+REM Mixed quotes and variables
+CALL "%PROGRAM_FILES%\\app.exe" arg1 arg2
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            assert "E014" not in rule_codes  # Should not trigger for environment variables
+        finally:
+            os.unlink(temp_file)
+
+    def test_call_label_still_triggers_e014(self) -> None:
+        """Test that actual label calls without colon still trigger E014."""
+        content = """@echo off
+CALL mylabel
+GOTO :EOF
+:mylabel
+ECHO In subroutine
+GOTO :EOF
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            assert "E014" in rule_codes  # Should still trigger for actual labels
+        finally:
+            os.unlink(temp_file)
+
     def test_goto_eof_without_colon(self) -> None:
         """Test E015: Missing colon in GOTO EOF statement."""
         content = """@echo off
@@ -1120,6 +1158,42 @@ echo \"\"\"incomplete triple quote
         finally:
             os.unlink(temp_file)
 
+    def test_e028_start_empty_title_no_error(self) -> None:
+        """Test E028: START with empty window title should not trigger error."""
+        content = """@ECHO OFF
+START "" CMD /K %@RUNAPP% %*
+START "" program.exe
+START "" notepad.exe file.txt
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            # Should not contain E028 for legitimate START "" patterns
+            assert "E028" not in rule_codes
+        finally:
+            os.unlink(temp_file)
+
+    def test_e028_legitimate_empty_string_patterns(self) -> None:
+        """Test E028: Legitimate empty string patterns should not trigger error."""
+        content = """@ECHO OFF
+REM Comparison operators with empty strings
+IF "%VAR%"=="" echo Empty
+IF NOT "%VAR%"=="" echo Not empty
+IF "%VAR%" neq "" echo Not equal
+IF "%VAR%" equ "" echo Equal
+REM START with empty title
+START "" notepad.exe
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            # Should not contain E028 for these legitimate patterns
+            assert "E028" not in rule_codes
+        finally:
+            os.unlink(temp_file)
+
     def test_e029_seta_expression_errors(self) -> None:
         """Test E029: Complex SET /A expression errors."""
         content = """@ECHO OFF
@@ -1398,3 +1472,681 @@ ping -n 5 localhost >nul
 
         finally:
             os.unlink(temp_file)
+
+    def test_sec014_subroutine_parameters_no_false_positive(self) -> None:
+        """Test SEC014: Parameters in subroutines should not be flagged as unescaped user input."""
+        # Reproduce the issue reported on GitHub:
+        # The command "SET @V1=%2& IF DEFINED @V1 SET @V1=!@V1:"=!" is part of a subroutine
+        # and should NOT trigger SEC014 because %2 refers to a subroutine parameter, not user input
+        content = """@ECHO OFF
+SETLOCAL ENABLEDELAYEDEXPANSION
+
+REM Main script
+CALL :MySubroutine arg1 arg2
+GOTO :EOF
+
+:MySubroutine
+REM This line should NOT trigger SEC014 because we're in a subroutine
+SET @V1=%2& IF DEFINED @V1 SET @V1=!@V1:"=!
+ECHO Subroutine parameter: !@V1!
+GOTO :EOF
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            # SEC014 should NOT be triggered for parameters used in subroutines
+            assert "SEC014" not in rule_codes, (
+                "SEC014 should not be triggered for subroutine parameters. "
+                f"Found issues: {[i for i in issues if i.rule.code == 'SEC014']}"
+            )
+        finally:
+            os.unlink(temp_file)
+
+    def test_sec014_main_script_parameters_should_trigger(self) -> None:
+        """Test SEC014: Parameters in main script SHOULD be flagged as potential user input."""
+        # Parameters used in the main script (before any labels) should still trigger SEC014
+        content = """@ECHO OFF
+REM Main script - parameters here are direct user input
+SET VAR=%1& ECHO %VAR%
+GOTO :EOF
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            # SEC014 SHOULD be triggered for parameters in main script
+            assert (
+                "SEC014" in rule_codes
+            ), "SEC014 should be triggered for parameters in main script"
+        finally:
+            os.unlink(temp_file)
+
+    def test_e030_for_command_string_no_false_positive(self) -> None:
+        """Test E030: Carets in FOR command strings should NOT be flagged as improper escaping."""
+        # Reproduces the issue found in real batch files where FOR /F IN ('command 2^>NUL')
+        # was incorrectly flagged as E030
+        content = """@ECHO OFF
+REM Test valid caret usage in FOR command strings
+FOR /F "tokens=*" %%a IN ('dir /b 2^>NUL') DO echo %%a
+SET X=1& FOR /F "DELIMS=" %%d IN ('DATEINFO -t START -n END -q 2^>NUL') DO SET Y=%%d
+
+REM Test ECHO with ASCII art (also should not trigger E030)
+ECHO Test^|Test
+
+REM Test invalid usage that SHOULD trigger E030
+IF EXIST file.txt^>NUL (
+    echo Found
+)
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            e030_issues = [issue for issue in issues if issue.rule.code == "E030"]
+            # Should only have 1 E030 issue on line 10 (the IF statement)
+            assert len(e030_issues) == 1, (
+                f"Expected 1 E030 issue, but found {len(e030_issues)}. "
+                f"Issues: {[(i.line_number, i.context) for i in e030_issues]}"
+            )
+            # Verify it's on the correct line (the IF statement with improper caret)
+            assert e030_issues[0].line_number == 10, (
+                f"E030 should be on line 10 (IF statement), "
+                f"but was on line {e030_issues[0].line_number}"
+            )
+        finally:
+            os.unlink(temp_file)
+
+    def test_w024_deprecated_commands_wmic(self) -> None:
+        """Test W024: WMIC command should be flagged as deprecated."""
+        content = """@ECHO OFF
+WMIC os get caption
+wmic process list
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            w024_issues = [issue for issue in issues if issue.rule.code == "W024"]
+            # Should detect both WMIC usages
+            assert "W024" in rule_codes, "W024 should be triggered for WMIC command"
+            assert len(w024_issues) == 2, f"Expected 2 W024 issues, got {len(w024_issues)}"
+            # Check context mentions WMIC
+            for issue in w024_issues:
+                assert (
+                    "WMIC" in issue.context.upper()
+                ), f"Context should mention WMIC: {issue.context}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_w024_deprecated_commands_cacls(self) -> None:
+        """Test W024: CACLS command should be flagged as deprecated."""
+        content = """@ECHO OFF
+CACLS file.txt /E /G user:F
+cacls folder /T
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            w024_issues = [issue for issue in issues if issue.rule.code == "W024"]
+            assert "W024" in rule_codes, "W024 should be triggered for CACLS command"
+            assert len(w024_issues) == 2, f"Expected 2 W024 issues, got {len(w024_issues)}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_w024_deprecated_commands_bitsadmin(self) -> None:
+        """Test W024: BITSADMIN command should be flagged as deprecated."""
+        content = """@ECHO OFF
+BITSADMIN /transfer myDownload http://example.com/file.zip c:\\temp\\file.zip
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            assert "W024" in rule_codes, "W024 should be triggered for BITSADMIN command"
+        finally:
+            os.unlink(temp_file)
+
+    def test_w024_deprecated_commands_winrm(self) -> None:
+        """Test W024: WINRM command should be flagged as deprecated."""
+        content = """@ECHO OFF
+WINRM quickconfig
+winrm get winrm/config
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            w024_issues = [issue for issue in issues if issue.rule.code == "W024"]
+            assert "W024" in rule_codes, "W024 should be triggered for WINRM command"
+            assert len(w024_issues) == 2, f"Expected 2 W024 issues, got {len(w024_issues)}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_w024_deprecated_commands_nbtstat(self) -> None:
+        """Test W024: NBTSTAT command should be flagged as deprecated."""
+        content = """@ECHO OFF
+NBTSTAT -n
+nbtstat -c
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            w024_issues = [issue for issue in issues if issue.rule.code == "W024"]
+            assert "W024" in rule_codes, "W024 should be triggered for NBTSTAT command"
+            assert len(w024_issues) == 2, f"Expected 2 W024 issues, got {len(w024_issues)}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_w024_deprecated_commands_net_send(self) -> None:
+        """Test W024: NET SEND command should be flagged as deprecated."""
+        content = """@ECHO OFF
+NET SEND computer "Hello"
+net send * "Broadcast message"
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            w024_issues = [issue for issue in issues if issue.rule.code == "W024"]
+            assert "W024" in rule_codes, "W024 should be triggered for NET SEND command"
+            assert len(w024_issues) == 2, f"Expected 2 W024 issues, got {len(w024_issues)}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_w024_deprecated_at_command(self) -> None:
+        """Test W024: AT command should be flagged as deprecated."""
+        content = """@ECHO OFF
+AT 14:00 script.bat
+at \\\\computer 10:00 backup.bat
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            w024_issues = [issue for issue in issues if issue.rule.code == "W024"]
+            assert "W024" in rule_codes, "W024 should be triggered for AT command"
+            assert len(w024_issues) == 2, f"Expected 2 W024 issues, got {len(w024_issues)}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_e034_removed_commands_caspol(self) -> None:
+        """Test E034: CASPOL command should be flagged as removed."""
+        content = """@ECHO OFF
+CASPOL -m -ag 1 -url file://c:\\temp\\* FullTrust
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            assert "E034" in rule_codes, "E034 should be triggered for CASPOL command"
+            e034_issues = [issue for issue in issues if issue.rule.code == "E034"]
+            assert "CASPOL" in e034_issues[0].context.upper()
+        finally:
+            os.unlink(temp_file)
+
+    def test_e034_removed_commands_diskcomp(self) -> None:
+        """Test E034: DISKCOMP command should be flagged as removed."""
+        content = """@ECHO OFF
+DISKCOMP A: B:
+diskcomp c:\\disk1 d:\\disk2
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            e034_issues = [issue for issue in issues if issue.rule.code == "E034"]
+            assert "E034" in rule_codes, "E034 should be triggered for DISKCOMP command"
+            assert len(e034_issues) == 2, f"Expected 2 E034 issues, got {len(e034_issues)}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_e034_removed_commands_append(self) -> None:
+        """Test E034: APPEND command should be flagged as removed."""
+        content = """@ECHO OFF
+APPEND C:\\DATA
+append /X:OFF
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            e034_issues = [issue for issue in issues if issue.rule.code == "E034"]
+            assert "E034" in rule_codes, "E034 should be triggered for APPEND command"
+            assert len(e034_issues) == 2, f"Expected 2 E034 issues, got {len(e034_issues)}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_e034_removed_commands_browstat(self) -> None:
+        """Test E034: BROWSTAT command should be flagged as removed."""
+        content = """@ECHO OFF
+BROWSTAT status
+browstat view
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            e034_issues = [issue for issue in issues if issue.rule.code == "E034"]
+            assert "E034" in rule_codes, "E034 should be triggered for BROWSTAT command"
+            assert len(e034_issues) == 2, f"Expected 2 E034 issues, got {len(e034_issues)}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_e034_removed_commands_inuse(self) -> None:
+        """Test E034: INUSE command should be flagged as removed."""
+        content = """@ECHO OFF
+INUSE file.dll /Y
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            assert "E034" in rule_codes, "E034 should be triggered for INUSE command"
+        finally:
+            os.unlink(temp_file)
+
+    def test_e034_removed_commands_net_print(self) -> None:
+        """Test E034: NET PRINT command should be flagged as removed (but NET itself is ok)."""
+        content = """@ECHO OFF
+REM NET PRINT is removed
+NET PRINT \\\\computer\\printer file.txt
+net print \\\\server\\queue
+
+REM But other NET commands are fine
+NET USE Z: \\\\server\\share
+NET VIEW \\\\computer
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            e034_issues = [issue for issue in issues if issue.rule.code == "E034"]
+            # Should only flag NET PRINT, not other NET commands
+            assert "E034" in rule_codes, "E034 should be triggered for NET PRINT command"
+            assert (
+                len(e034_issues) == 2
+            ), f"Expected 2 E034 issues (NET PRINT only), got {len(e034_issues)}"
+            # Verify it's specifically for NET PRINT
+            for issue in e034_issues:
+                assert (
+                    "PRINT" in issue.context.upper()
+                ), f"Context should mention PRINT: {issue.context}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_e034_removed_commands_diskcopy(self) -> None:
+        """Test E034: DISKCOPY command should be flagged as removed."""
+        content = """@ECHO OFF
+DISKCOPY A: B:
+diskcopy c:\\source d:\\dest
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            e034_issues = [issue for issue in issues if issue.rule.code == "E034"]
+            assert "E034" in rule_codes, "E034 should be triggered for DISKCOPY command"
+            assert len(e034_issues) == 2, f"Expected 2 E034 issues, got {len(e034_issues)}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_e034_removed_commands_streams(self) -> None:
+        """Test E034: STREAMS command should be flagged as removed."""
+        content = """@ECHO OFF
+STREAMS -s file.txt
+streams -d *.doc
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            e034_issues = [issue for issue in issues if issue.rule.code == "E034"]
+            assert "E034" in rule_codes, "E034 should be triggered for STREAMS command"
+            assert len(e034_issues) == 2, f"Expected 2 E034 issues, got {len(e034_issues)}"
+        finally:
+            os.unlink(temp_file)
+
+    def test_xcopy_not_deprecated(self) -> None:
+        """Test that XCOPY is NOT flagged as deprecated (per requirements)."""
+        content = """@ECHO OFF
+XCOPY source dest /E /Y
+xcopy *.txt backup\\
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            # XCOPY should NOT trigger W024 or E034
+            assert "W024" not in rule_codes, "XCOPY should NOT be flagged as deprecated"
+            assert "E034" not in rule_codes, "XCOPY should NOT be flagged as removed"
+        finally:
+            os.unlink(temp_file)
+
+    def test_net_command_not_deprecated_without_send(self) -> None:
+        """Test that NET commands (other than NET SEND and NET PRINT) are not flagged."""
+        content = """@ECHO OFF
+NET USE Z: \\\\server\\share
+NET VIEW \\\\computer
+NET USER username
+NET STOP servicename
+net start servicename
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            rule_codes = [issue.rule.code for issue in issues]
+            # These NET commands should NOT trigger W024 or E034
+            assert (
+                "W024" not in rule_codes
+            ), "NET (without SEND) should NOT be flagged as deprecated"
+            assert "E034" not in rule_codes, "NET (without PRINT) should NOT be flagged as removed"
+        finally:
+            os.unlink(temp_file)
+
+    def test_deprecated_and_removed_commands_comprehensive(self) -> None:
+        """Test comprehensive check of multiple deprecated and removed commands in one file."""
+        content = """@ECHO OFF
+REM Deprecated commands (should trigger W024)
+WMIC os get caption
+CACLS file.txt /E
+BITSADMIN /transfer test http://test.com/file.zip c:\\file.zip
+WINRM quickconfig
+NBTSTAT -n
+NET SEND computer "message"
+AT 14:00 task.bat
+DPATH C:\\DATA
+KEYS
+
+REM Removed commands (should trigger E034)
+CASPOL -m -ag 1
+DISKCOMP A: B:
+APPEND C:\\DATA
+BROWSTAT status
+INUSE file.dll
+NET PRINT \\\\server\\printer file.txt
+DISKCOPY A: B:
+STREAMS -s file.txt
+
+REM Valid commands (should NOT trigger W024 or E034)
+NET USE Z: \\\\server\\share
+XCOPY source dest /E
+ROBOCOPY source dest /E
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            w024_issues = [issue for issue in issues if issue.rule.code == "W024"]
+            e034_issues = [issue for issue in issues if issue.rule.code == "E034"]
+
+            # Should have 9 deprecated command warnings
+            assert len(w024_issues) == 9, (
+                f"Expected 9 W024 issues (deprecated commands), got {len(w024_issues)}. "
+                f"Issues: {[(i.line_number, i.context) for i in w024_issues]}"
+            )
+
+            # Should have 8 removed command errors
+            assert len(e034_issues) == 8, (
+                f"Expected 8 E034 issues (removed commands), got {len(e034_issues)}. "
+                f"Issues: {[(i.line_number, i.context) for i in e034_issues]}"
+            )
+        finally:
+            os.unlink(temp_file)
+
+    def test_w035_tokens_star_is_valid(self) -> None:
+        """Test W035: FOR /F with tokens=* should NOT trigger delimiter warning."""
+        # GitHub issue: tokens=* is a valid pattern that doesn't need explicit delimiters
+        # tokens=* means "take the entire line without tokenization"
+        content = """@ECHO OFF
+REM tokens=* is valid - should NOT trigger W035
+FOR /F "tokens=*" %%a IN ('dir /b') DO echo %%a
+FOR /F "TOKENS=*" %%b IN ('type file.txt') DO SET LINE=%%b
+
+REM tokens=* with SKIP is also valid - should NOT trigger W035
+FOR /F "SKIP=1 TOKENS=*" %%c IN ('dir /b') DO echo %%c
+
+REM Other tokens= patterns without delims= SHOULD trigger W035
+FOR /F "tokens=1,2" %%d IN ('type data.txt') DO echo %%d
+FOR /F "tokens=3" %%e IN ('dir') DO SET VALUE=%%e
+
+REM tokens= with explicit delims= should NOT trigger W035
+FOR /F "tokens=1,2 delims=," %%f IN ('type csv.txt') DO echo %%f
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            w035_issues = [issue for issue in issues if issue.rule.code == "W035"]
+
+            # Should have exactly 2 W035 issues (lines 10 and 11)
+            # Lines 3, 4, and 7 use tokens=* and should NOT trigger
+            # Line 14 has explicit delims= and should NOT trigger
+            assert len(w035_issues) == 2, (
+                f"Expected 2 W035 issues, got {len(w035_issues)}. "
+                f"Issues on lines: {[i.line_number for i in w035_issues]}"
+            )
+
+            # Verify the warnings are on the correct lines
+            warning_lines = sorted([i.line_number for i in w035_issues])
+            assert warning_lines == [
+                10,
+                11,
+            ], f"Expected W035 on lines [10, 11], got {warning_lines}"
+        finally:
+            os.unlink(temp_file)
+
+
+class TestInlineSuppressions:
+    """Test cases for inline lint suppression comments."""
+
+    def create_temp_batch_file(self, content: str) -> str:
+        """Helper method to create a temporary batch file."""
+        with tempfile.NamedTemporaryFile(
+            mode="w", suffix=".bat", delete=False, encoding="utf-8"
+        ) as temp_file:
+            temp_file.write(content)
+            return temp_file.name
+
+    def test_lint_ignore_line_all_rules(self) -> None:
+        """Test LINT:IGNORE-LINE suppressing all rules on same line."""
+        content = """@ECHO OFF
+echo lowercase  REM LINT:IGNORE-LINE
+EXIT /b 0
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Line 2 should have no issues because of LINT:IGNORE-LINE
+            line2_issues = [i for i in issues if i.line_number == 2]
+            assert len(line2_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_lint_ignore_line_specific_rules(self) -> None:
+        """Test LINT:IGNORE-LINE with specific rule codes."""
+        content = """@ECHO OFF
+echo lowercase  REM LINT:IGNORE-LINE STY001
+EXIT /b 0
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should suppress STY001 but not other rules
+            sty001_line2 = [i for i in issues if i.line_number == 2 and i.rule.code == "STY001"]
+            assert len(sty001_line2) == 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_lint_ignore_next_line_all_rules(self) -> None:
+        """Test LINT:IGNORE suppressing all rules on next line."""
+        content = """@ECHO OFF
+REM LINT:IGNORE
+echo lowercase
+EXIT /b 0
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Line 3 should have no issues because of LINT:IGNORE on line 2
+            line3_issues = [i for i in issues if i.line_number == 3]
+            assert len(line3_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+
+class TestNestedForLoops:
+    """Test cases for nested FOR loop detection."""
+
+    def create_temp_batch_file(self, content: str) -> str:
+        """Helper method to create a temporary batch file."""
+        with tempfile.NamedTemporaryFile(
+            mode="w", suffix=".bat", delete=False, encoding="utf-8"
+        ) as temp_file:
+            temp_file.write(content)
+            return temp_file.name
+
+    def test_nested_for_loop_without_call(self) -> None:
+        """Test W039: Nested FOR loop without CALL :subroutine."""
+        content = """@ECHO OFF
+FOR %%i IN (1 2 3) DO (
+    FOR %%j IN (a b c) DO (
+        ECHO %%i %%j
+    )
+)
+EXIT /b 0
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should detect nested FOR loop issue
+            w039_issues = [i for i in issues if i.rule.code == "W039"]
+            assert len(w039_issues) > 0
+        finally:
+            os.unlink(temp_file)
+
+
+class TestRestartLogic:
+    """Test cases for restart/retry logic without limits (SEC016)."""
+
+    def create_temp_batch_file(self, content: str) -> str:
+        """Helper method to create a temporary batch file."""
+        with tempfile.NamedTemporaryFile(
+            mode="w", suffix=".bat", delete=False, encoding="utf-8"
+        ) as temp_file:
+            temp_file.write(content)
+            return temp_file.name
+
+    def test_restart_without_limit(self) -> None:
+        """Test SEC016: Restart logic without failure attempt limits."""
+        content = """@ECHO OFF
+:retry_loop
+net start MyService
+IF ERRORLEVEL 1 GOTO retry_loop
+EXIT /b 0
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should detect restart without limit
+            sec016_issues = [i for i in issues if i.rule.code == "SEC016"]
+            assert len(sec016_issues) > 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_restart_with_counter(self) -> None:
+        """Test restart logic with counter (safe pattern)."""
+        content = """@ECHO OFF
+SET counter=0
+:retry_loop
+SET /A counter+=1
+IF %counter% GTR 5 GOTO failed
+net start MyService
+IF ERRORLEVEL 1 GOTO retry_loop
+EXIT /b 0
+:failed
+ECHO Max attempts reached
+EXIT /b 1
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should NOT detect SEC016 when counter is present
+            sec016_issues = [i for i in issues if i.rule.code == "SEC016"]
+            assert len(sec016_issues) == 0
+        finally:
+            os.unlink(temp_file)
+
+
+class TestPerformanceRules:
+    """Test cases for performance rules."""
+
+    def create_temp_batch_file(self, content: str) -> str:
+        """Helper method to create a temporary batch file."""
+        with tempfile.NamedTemporaryFile(
+            mode="w", suffix=".bat", delete=False, encoding="utf-8"
+        ) as temp_file:
+            temp_file.write(content)
+            return temp_file.name
+
+    def test_p017_redundant_file_checks(self) -> None:
+        """Test P017: Multiple redundant file existence checks."""
+        content = """@ECHO OFF
+IF EXIST test.txt ECHO File exists
+IF EXIST test.txt ECHO Checking again
+IF EXIST test.txt ECHO And again
+EXIT /b 0
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should detect redundant file checks
+            p017_issues = [i for i in issues if i.rule.code == "P017"]
+            assert len(p017_issues) > 0
+        finally:
+            os.unlink(temp_file)
+
+    def test_p020_redundant_echo_off(self) -> None:
+        """Test P020: @echo off appearing after line 1."""
+        content = """REM Header comment
+@ECHO OFF
+ECHO This is after echo off
+EXIT /b 0
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should detect @echo off not on first line
+            p020_issues = [i for i in issues if i.rule.code == "P020"]
+            assert len(p020_issues) > 0
+        finally:
+            os.unlink(temp_file)
+
+
+class TestStyleRulesExtended:
+    """Test cases for extended style rules."""
+
+    def create_temp_batch_file(self, content: str) -> str:
+        """Helper method to create a temporary batch file."""
+        with tempfile.NamedTemporaryFile(
+            mode="w", suffix=".bat", delete=False, encoding="utf-8"
+        ) as temp_file:
+            temp_file.write(content)
+            return temp_file.name
+
+    def test_s017_inconsistent_variable_case(self) -> None:
+        """Test S017: Inconsistent variable name casing."""
+        content = """@ECHO OFF
+SET MyVar=value1
+SET MYVAR=value2
+SET myvar=value3
+ECHO %MyVar% %MYVAR% %myvar%
+EXIT /b 0
+"""
+        temp_file = self.create_temp_batch_file(content)
+        try:
+            issues = lint_batch_file(temp_file)
+            # Should detect inconsistent variable casing
+            s017_issues = [i for i in issues if i.rule.code == "S017"]
+            assert len(s017_issues) > 0
+        finally:
+            os.unlink(temp_file)
